{demo of using Xon/Xoff to implement flow control, writing a file to eeprom
@Tracy Allen, EME Systems LLC
Accepts a text file from CoolTerm
File EOF char is here defined as "*"
File is saved into the high eeprom, an 24LC512 tied to the system pins scl=28 sda=29
After EOF is received, the demo regurgitates the file and prints it on the CoolTerm screen.
-- enable Xon in the CoolTerm terminal options, 19200 baud.
-- use CoolTerm  Connection/Sendfile.   File must end with EOF="*"
}

CON
  _clkmode = xtal1 + pll16x                '' clkfreq=40MHz
  _xinfreq = 5_000_000
  CLKFREQ0 = _xinfreq * (_clkmode >> 6 #> 1)   ' only for xtal based clk modes

BAUD = 19200
XON = 17        ' xmission on
XOFF = 19       ' xmission off
EOF = "*"
CR = 13

PAGESIZE = 128  ' note set FDS4port for 200 char receive buffer on port 0



OBJ
  uarts : "fullDuplexSerial4port"    ' set port 0 receive buffer for 200 bytes
  i2c : "i2c_1bus"             ' demo uses 24LC512 on system pins scl=28 sda=29

VAR
  byte mybuf[130]   ' will use 128 bytes for eeprom pages

PUB Main | idx, char, eeAdrs
  StartUarts
  i2c.init(28, 29)  ' on system pins
  eeAdrs := 32768   ' start of auxiliary eeprom
  uarts.str(0,string(13,"top of XonXoff probe, send me a text file",13))
  repeat
    uarts.tx(0, XON)
    idx~
    repeat
      if (char := uarts.rxCheck(0)) > -1
        mybuf[idx++] := char
    until idx == PAGESIZE or char == EOF   ' be sure the buffer is sized "enough" larger than the page size
    uarts.tx(0, XOFF)
    ShowHowFull
    i2c.WritePage($A0, -eeAdrs, @myBuf, 128)    ' negative address means 16 bits
    eeAdrs += 128
  until char == EOF
  uarts.str(0,string(CR,"EOF received",CR))
  Pause(1000)
  ShowFile
  uarts.str(0,string(CR,"showFile done",CR))


PRI StartUarts
  uarts.Init  ' stop the ports
  uarts.AddPort(0,31,30,-1,-1,0,0,BAUD) ' debug on prop plug pins
  uarts.Start
  waitcnt(clkfreq/10+cnt)

PRI ShowFile | eeAdrs, char
  eeAdrs := 32768
  char := i2c.readByte($A0, -eeAdrs)         '
  repeat
    uarts.tx(0,char)
    char := i2c.readByte($A0, i2c#NEXTUP)   ' next byte
  until char == EOF

PRI ShowHowFull
  uarts.dec(0,uarts.rxHowFull(0))

PRI Pause(ms)
  waitcnt(clkfreq/1000*ms+cnt)


